#include <stdio.h>
#include <sys/types.h>
#include <pwd.h>
#include<sys/stat.h>

#if defined(SOLARIS2) || defined(SYSV) || defined(AIX)
# include <string.h>
# include "flock.h"
# define index(s, c)	strchr(s, c)
#else
# include <strings.h>
#endif

#if defined(SOLARIS2) || defined(UNIXWARE) || defined(AIX) || defined(PTX) \
	|| defined(AUX) || defined(POPSCO) || defined(OSF1) || defined(ULTRIX)
# include <fcntl.h>
#else
# include <sys/file.h>
#endif

#ifdef GDBM
#include <gdbm.h>
#else
#include <ndbm.h>
#endif

#include "popper.h"

/* 
 *  auth  : optional command ; Not supported.
 */

int pop_auth (p)
POP     *   p;
{
    /*  Tell the user that this command is not supported */
    return (pop_msg(p,POP_FAILURE,"This command is not supported yet"));
}

